@@ -59,6 +59,9 @@ information.
 """
 
 
+from __future__ import annotations
+
+
 class FunctionalGroup:
     """
     An abstract base class for functional groups.
@@ -68,6 +71,58 @@ class FunctionalGroup:
     :class:`.ConstructedMolecule` construction, as well as specify
     which atoms of the building block should be used for positioning.
 
+    *Should I use* :meth:`.with_ids` *or* :meth:`.with_atoms` *?*
+
+    That depends on your use case, however, it is generally better to
+    default to :meth:`.with_ids` unless you need to actually change
+    the atoms held by the functional group. This is because
+    :meth:`.with_ids` preserves the most-derived type of the functional
+    group, while :meth:`.with_atoms` does not. To give an example
+
+    .. testcode:: with-atoms-vs-with-ids
+
+        import stk
+
+        bromo = stk.Bromo(
+            bromine=stk.Br(0),
+            atom=stk.C(1),
+            bonders=(stk.C(1), ),
+            deleters=(stk.Br(0), ),
+        )
+
+        bromo2 = bromo.with_ids({
+            0: 10,
+            1: 100,
+        })
+        # bromo2 is still a Bromo functional group.
+        assert isinstance(bromo2, stk.Bromo)
+
+        not_bromo = bromo.with_atoms({
+            0: stk.Br(10),
+            1: stk.C(100),
+        })
+        # not_bromo is not a Bromo functional gorup.
+        assert not isinstance(not_bromo, stk.Bromo)
+        # However, it is still an instance of FunctionalGroup,
+        assert isinstance(not_bromo, stk.FunctionalGroup)
+        # and of GenericFunctionalGroup
+        assert isinstance(not_bromo, stk.GenericFunctionalGroup)
+
+    The reason that :meth:`.with_atoms` does not produce a
+    :class:`.Bromo` instance is to avoid the following pitfall
+
+    .. code-block:: python
+
+        pitfall = bromo.with_atoms({
+            0: stk.F(10),
+            1: stk.C(100),
+        })
+        # If with_atoms() returned a Bromo instance then you could
+        # call get_bromine() on it, but it would hold a F atom!
+        this_is_a_fluorine = pitfall.get_bromine()
+
+    *Why would I want to implement a new subclass?*
+
     The most common reason you would want to implement a new
     :class:`.FunctionalGroup` subclass, is because you want to
     customize the construction of a :class:`.ConstructedMolecule`.
@@ -86,91 +141,94 @@ class FunctionalGroup:
     :class:`~.polymer.linear.linear.Linear`, and your custom
     modification will take place.
 
-    See Also
-    --------
-    :mod:`.functional_group_factory`
-        Used for automated creation of :class:`.FunctionalGroup`
-        instances.
+    See Also:
 
-    Notes
-    -----
-    You might notice that some of the methods of this abstract base
-    class are implemented. This is purely for convenience when
-    implementing subclasses. The implemented public methods are simply
-    default implementations, which can be safely ignored
-    or overridden, when implementing subclasses. Any private methods
-    are implementation details of these default implementations.
+        :mod:`.functional_group_factory`
+            Used for automated creation of :class:`.FunctionalGroup`
+            instances.
 
-    Examples
-    --------
-    *Subclass Implementation*
+    Notes:
 
-    The source code of the subclasses, listed in
-    :mod:`.functional_group`, can serve as good examples.
+        You might notice that some of the methods of this abstract base
+        class are implemented. This is purely for convenience when
+        implementing subclasses. The implemented public methods are
+        simply default implementations, which can be safely ignored
+        or overridden, when implementing subclasses. Any private
+        methods are implementation details of these default
+        implementations.
 
-    *Changing the Atoms of a Functional Group*
+    Examples:
 
-    You want to substitute the atoms in the functional group for other
-    atoms. You can do this by using :meth:`.with_atoms` to create a
-    clone of the functional group, which holds the replacement atoms
+        *Subclass Implementation*
 
-    .. testcode:: changing-the-atoms-of-a-functional-group
+        The source code of the subclasses, listed in
+        :mod:`.functional_group`, can serve as good examples.
 
-        import stk
+        *Changing the Atoms of a Functional Group*
 
-        c, n, h1, h2 = stk.C(0), stk.N(1), stk.H(2), stk.H(3)
-        amine = stk.PrimaryAmino(
-            nitrogen=n,
-            hydrogen1=h1,
-            hydrogen2=h2,
-            atom=c,
-            bonders=(n, ),
-            deleters=(h1, h2),
-        )
+        You want to substitute the atoms in the functional group for
+        other atoms. You can do this by using :meth:`.with_atoms` to
+        create a clone of the functional group, which holds the
+        replacement atoms
 
-        n20 = stk.N(20)
-        h100 = stk.H(100)
+        .. testcode:: changing-the-atoms-of-a-functional-group
 
-        # amine_clone is a clone of amine, except that instead of
-        # holding n, amine_clone holds n20, and instead of holding h1
-        # amine_clone holds h100. amine_clone continues to hold h2.
-        amine_clone = amine.with_atoms({
-            n.get_id(): n20,
-            h1.get_id(): h100,
-        })
+            import stk
 
-    .. testcode:: changing-the-atoms-of-a-functional-group
-        :hide:
+            c, n, h1, h2 = stk.C(0), stk.N(1), stk.H(2), stk.H(3)
+            amine = stk.PrimaryAmino(
+                nitrogen=n,
+                hydrogen1=h1,
+                hydrogen2=h2,
+                atom=c,
+                bonders=(n, ),
+                deleters=(h1, h2),
+            )
 
-        _atoms = set(amine.get_atom_ids())
-        assert n.get_id() in _atoms
-        assert n20.get_id() not in _atoms
-        assert h1.get_id() in _atoms
-        assert h100.get_id()  not in _atoms
+            n20 = stk.N(20)
+            h100 = stk.H(100)
 
-        _clone_atoms = set(amine_clone.get_atom_ids())
-        assert n.get_id() not in _clone_atoms
-        assert n20.get_id() in _clone_atoms
-        assert h1.get_id() not in _clone_atoms
-        assert h100.get_id() in _clone_atoms
+            # amine_clone is a clone of amine, except that instead of
+            # holding n, amine_clone holds n20, and instead of holding
+            # h1  amine_clone holds h100. amine_clone continues to hold
+            # h2.
+            amine_clone = amine.with_atoms({
+                n.get_id(): n20,
+                h1.get_id(): h100,
+            })
 
-        _bonders = set(amine.get_bonder_ids())
-        assert n.get_id() in _bonders
-        assert n20.get_id() not in _bonders
+        .. testcode:: changing-the-atoms-of-a-functional-group
+            :hide:
 
-        _clone_bonders = set(amine_clone.get_bonder_ids())
-        assert n.get_id() not in _clone_bonders
-        assert n20.get_id() in _clone_bonders
+            _atoms = set(amine.get_atom_ids())
+            assert n.get_id() in _atoms
+            assert n20.get_id() not in _atoms
+            assert h1.get_id() in _atoms
+            assert h100.get_id()  not in _atoms
 
-        _deleters = set(amine.get_deleter_ids())
-        assert h1.get_id() in _deleters
-        assert h2.get_id() in _deleters
-        assert h100.get_id() not in _deleters
+            _clone_atoms = set(amine_clone.get_atom_ids())
+            assert n.get_id() not in _clone_atoms
+            assert n20.get_id() in _clone_atoms
+            assert h1.get_id() not in _clone_atoms
+            assert h100.get_id() in _clone_atoms
 
-        _clone_deleters = set(amine_clone.get_deleter_ids())
-        assert h1.get_id() not in _clone_deleters
-        assert h2.get_id() in _clone_deleters
-        assert h100.get_id() in _clone_deleters
+            _bonders = set(amine.get_bonder_ids())
+            assert n.get_id() in _bonders
+            assert n20.get_id() not in _bonders
+
+            _clone_bonders = set(amine_clone.get_bonder_ids())
+            assert n.get_id() not in _clone_bonders
+            assert n20.get_id() in _clone_bonders
+
+            _deleters = set(amine.get_deleter_ids())
+            assert h1.get_id() in _deleters
+            assert h2.get_id() in _deleters
+            assert h100.get_id() not in _deleters
+
+            _clone_deleters = set(amine_clone.get_deleter_ids())
+            assert h1.get_id() not in _clone_deleters
+            assert h2.get_id() in _clone_deleters
+            assert h100.get_id() in _clone_deleters
 
     """
 
@@ -257,23 +315,6 @@ class FunctionalGroup:
 
         yield from (a.get_id() for a in self._core_atoms)
 
-    def _with_atoms(self, atom_map):
-        """
-        Modify the functional group.
-
-        """
-
-        self._atoms = tuple(
-            atom_map.get(a.get_id(), a) for a in self._atoms
-        )
-        self._placers = tuple(
-            atom_map.get(a.get_id(), a) for a in self._placers
-        )
-        self._core_atoms = tuple(
-            atom_map.get(a.get_id(), a) for a in self._core_atoms
-        )
-        return self
-
     def with_atoms(self, atom_map):
         """
         Return a clone holding different atoms.
@@ -294,7 +335,65 @@ class FunctionalGroup:
 
         """
 
-        return self.clone()._with_atoms(atom_map)
+        # The clone needs to be downcasted.
+        return FunctionalGroup(
+            atoms=tuple(
+                atom_map.get(a.get_id(), a) for a in self._atoms
+            ),
+            placers=tuple(
+                atom_map.get(a.get_id(), a) for a in self._placers
+            ),
+            core_atoms=tuple(
+                atom_map.get(a.get_id(), a) for a in self._core_atoms
+            ),
+        )
+
+    def _with_ids(self, id_map: dict[int, int]) -> FunctionalGroup:
+        self._atoms = tuple(
+            atom.with_id(
+                id=id_map.get(
+                    atom.get_id(),
+                    atom.get_id(),
+                ),
+            ) for atom in self._atoms
+        )
+        self._placers = tuple(
+            atom.with_id(
+                id=id_map.get(
+                    atom.get_id(),
+                    atom.get_id(),
+                ),
+            ) for atom in self._placers
+        )
+        self._core_atoms = tuple(
+            atom.with_id(
+                id=id_map.get(
+                    atom.get_id(),
+                    atom.get_id(),
+                ),
+            ) for atom in self._core_atoms
+        )
+        return self
+
+    def with_ids(self, id_map: dict[int, int]) -> FunctionalGroup:
+        """
+        Return a clone holding different atom ids.
+
+        Parameters:
+
+            id_map:
+                Maps the id of an atom in the functional group to the
+                new id the clone should hold. If the id of an atom in
+                the functional group is not found in `id_map`, the
+                atom will not be replaced in the clone.
+
+        Returns:
+
+            The clone.
+
+        """
+
+        return self.clone()._with_ids(id_map)
 
     def clone(self):
         """
